@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: edge.c,v 1.1.2.25 2003/07/30 11:50:45 guus Exp $
+    $Id: edge.c,v 1.1.2.27 2003/08/28 21:05:10 guus Exp $
 */
 
 #include "system.h"
@@ -58,15 +58,14 @@ void init_edges(void)
 {
 	cp();
 
-	edge_weight_tree =
-		avl_alloc_tree((avl_compare_t) edge_weight_compare, NULL);
+	edge_weight_tree = avl_alloc_tree((avl_compare_t) edge_weight_compare, NULL);
 }
 
 avl_tree_t *new_edge_tree(void)
 {
 	cp();
 
-	return avl_alloc_tree((avl_compare_t) edge_compare, NULL);
+	return avl_alloc_tree((avl_compare_t) edge_compare, (avl_action_t) free_edge);
 }
 
 void free_edge_tree(avl_tree_t *edge_tree)
@@ -89,12 +88,14 @@ edge_t *new_edge(void)
 {
 	cp();
 
-	return (edge_t *) xmalloc_and_zero(sizeof(edge_t));
+	return xmalloc_and_zero(sizeof(edge_t));
 }
 
 void free_edge(edge_t *e)
 {
 	cp();
+	
+	sockaddrfree(&e->address);
 
 	free(e);
 }
@@ -119,8 +120,8 @@ void edge_del(edge_t *e)
 	if(e->reverse)
 		e->reverse->reverse = NULL;
 
-	avl_delete(e->from->edge_tree, e);
 	avl_delete(edge_weight_tree, e);
+	avl_delete(e->from->edge_tree, e);
 }
 
 edge_t *lookup_edge(node_t *from, node_t *to)
@@ -147,9 +148,9 @@ void dump_edges(void)
 	logger(LOG_DEBUG, _("Edges:"));
 
 	for(node = node_tree->head; node; node = node->next) {
-		n = (node_t *) node->data;
+		n = node->data;
 		for(node2 = n->edge_tree->head; node2; node2 = node2->next) {
-			e = (edge_t *) node2->data;
+			e = node2->data;
 			address = sockaddr2hostname(&e->address);
 			logger(LOG_DEBUG, _(" %s to %s at %s options %lx weight %d"),
 				   e->from->name, e->to->name, address, e->options, e->weight);
