@@ -17,11 +17,12 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: meta.c,v 1.1.2.41 2003/08/12 14:48:12 guus Exp $
+    $Id: meta.c,v 1.1.2.48 2003/10/12 11:40:00 guus Exp $
 */
 
 #include "system.h"
 
+#include <openssl/err.h>
 #include <openssl/evp.h>
 
 #include "avl_tree.h"
@@ -46,7 +47,12 @@ bool send_meta(connection_t *c, const char *buffer, int length)
 			   c->name, c->hostname);
 
 	if(c->status.encryptout) {
-		EVP_EncryptUpdate(c->outctx, outbuf, &outlen, buffer, length);
+		result = EVP_EncryptUpdate(c->outctx, outbuf, &outlen, buffer, length);
+		if(!result || outlen != length) {
+			logger(LOG_ERR, _("Error while encrypting metadata to %s (%s): %s"),
+					c->name, c->hostname, ERR_error_string(ERR_get_error(), NULL));
+			return false;
+		}
 		bufp = outbuf;
 		length = outlen;
 	} else
@@ -80,7 +86,7 @@ void broadcast_meta(connection_t *from, const char *buffer, int length)
 	cp();
 
 	for(node = connection_tree->head; node; node = node->next) {
-		c = (connection_t *) node->data;
+		c = node->data;
 
 		if(c != from && c->status.active)
 			send_meta(c, buffer, length);
@@ -89,27 +95,13 @@ void broadcast_meta(connection_t *from, const char *buffer, int length)
 
 bool receive_meta(connection_t *c)
 {
-	int x;
-	socklen_t l = sizeof(x);
-	int oldlen, i;
-	int lenin, reqlen;
+	int oldlen, i, result;
+	int lenin, lenout, reqlen;
 	bool decrypted = false;
 	char inbuf[MAXBUFSIZE];
 
 	cp();
 
-	if(getsockopt(c->socket, SOL_SOCKET, SO_ERROR, &x, &l) < 0) {
-		logger(LOG_ERR, _("This is a bug: %s:%d: %d:%s %s (%s)"), __FILE__,
-			   __LINE__, c->socket, strerror(errno), c->name, c->hostname);
-		return false;
-	}
-
-	if(x) {
-		logger(LOG_ERR, _("Metadata socket error for %s (%s): %s"),
-			   c->name, c->hostname, strerror(x));
-		return false;
-	}
-
 	/* Strategy:
 	   - Read as much as possible from the TCP socket in one go.
 	   - Decrypt it.
@@ -137,11 +129,16 @@ bool receive_meta(connection_t *c)
 	oldlen = c->buflen;
 	c->buflen += lenin;
 
-	while(lenin) {
+	while(lenin > 0) {
 		/* Decrypt */
 
 		if(c->status.decryptin && !decrypted) {
-			EVP_DecryptUpdate(c->inctx, inbuf, &lenin, c->buffer + oldlen, lenin);
+			result = EVP_DecryptUpdate(c->inctx, inbuf, &lenout, c->buffer + oldlen, lenin);
+			if(!result || lenout != lenin) {
+				logger(LOG_ERR, _("Error while decrypting metadata from %s (%s): %s"),
+						c->name, c->hostname, ERR_error_string(ERR_get_error(), NULL));
+				return false;
+			}
 			memcpy(c->buffer + oldlen, inbuf, lenin);
 			decrypted = true;
 		}
@@ -153,7 +150,7 @@ bool receive_meta(connection_t *c)
 				receive_tcppacket(c, c->buffer, c->tcplen);
 
 				c->buflen -= c->tcplen;
-				lenin -= c->tcplen;
+				lenin -= c->tcplen - oldlen;
 				memmove(c->buffer, c->buffer + c->tcplen, c->buflen);
 				oldlen = 0;
 				c->tcplen = 0;
@@ -181,7 +178,7 @@ bool receive_meta(connection_t *c)
 				return false;
 
 			c->buflen -= reqlen;
-			lenin -= reqlen;
+			lenin -= reqlen - oldlen;
 			memmove(c->buffer, c->buffer + reqlen, c->buflen);
 			oldlen = 0;
 			continue;
