@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: process.c,v 1.1.2.71 2003/08/10 13:35:05 guus Exp $
+    $Id: process.c,v 1.1.2.76 2003/10/06 16:13:08 guus Exp $
 */
 
 #include "system.h"
@@ -58,19 +58,6 @@ static void memory_full(int size)
 
 /* Some functions the less gifted operating systems might lack... */
 
-#ifndef HAVE_FCLOSEALL
-static int fcloseall(void)
-{
-	fflush(stdin);
-	fflush(stdout);
-	fflush(stderr);
-	fclose(stdin);
-	fclose(stdout);
-	fclose(stderr);
-	return 0;
-}
-#endif
-
 #ifdef HAVE_MINGW
 extern char *identname;
 extern char *program_name;
@@ -82,7 +69,7 @@ static SERVICE_STATUS status = {0};
 static SERVICE_STATUS_HANDLE statushandle = 0;
 
 bool install_service(void) {
-	char command[4096] = "";
+	char command[4096] = "\"";
 	char **argp;
 	bool space;
 	SERVICE_DESCRIPTION description = {"Virtual Private Network daemon"};
@@ -94,11 +81,14 @@ bool install_service(void) {
 	}
 
 	if(!strchr(program_name, '\\')) {
-		GetCurrentDirectory(sizeof(command), command);
+		GetCurrentDirectory(sizeof(command) - 1, command + 1);
 		strncat(command, "\\", sizeof(command));
 	}
 
 	strncat(command, program_name, sizeof(command));
+
+	strncat(command, "\"", sizeof(command));
+
 	for(argp = g_argv + 1; *argp; argp++) {
 		space = strchr(*argp, ' ');
 		strncat(command, " ", sizeof(command));
@@ -166,24 +156,28 @@ DWORD WINAPI controlhandler(DWORD request, DWORD type, LPVOID boe, LPVOID bah) {
 	switch(request) {
 		case SERVICE_CONTROL_STOP:
 			logger(LOG_NOTICE, _("Got %s request"), "SERVICE_CONTROL_STOP");
-			running = false;
 			break;
 		case SERVICE_CONTROL_SHUTDOWN:
 			logger(LOG_NOTICE, _("Got %s request"), "SERVICE_CONTROL_SHUTDOWN");
-			running = false;
 			break;
 		default:
 			logger(LOG_WARNING, _("Got unexpected request %d"), request);
 			return ERROR_CALL_NOT_IMPLEMENTED;
 	}
 
-	if(!running) {
+	if(running) {
+		running = false;
 		status.dwWaitHint = 30000; 
 		status.dwCurrentState = SERVICE_STOP_PENDING; 
 		SetServiceStatus(statushandle, &status);
+		return NO_ERROR;
+	} else {
+		status.dwWaitHint = 0; 
+		status.dwCurrentState = SERVICE_STOPPED; 
+		SetServiceStatus(statushandle, &status);
+		exit(1);
 	}
 
-	return NO_ERROR;
 }
 
 VOID WINAPI run_service(DWORD argc, LPTSTR* argv)
@@ -247,7 +241,7 @@ bool init_service(void) {
 */
 static bool write_pidfile(void)
 {
-	int pid;
+	pid_t pid;
 
 	cp();
 
@@ -255,10 +249,10 @@ static bool write_pidfile(void)
 
 	if(pid) {
 		if(netname)
-			fprintf(stderr, _("A tincd is already running for net `%s' with pid %d.\n"),
-					netname, pid);
+			fprintf(stderr, _("A tincd is already running for net `%s' with pid %ld.\n"),
+					netname, (long)pid);
 		else
-			fprintf(stderr, _("A tincd is already running with pid %d.\n"), pid);
+			fprintf(stderr, _("A tincd is already running with pid %ld.\n"), (long)pid);
 		return false;
 	}
 
@@ -276,7 +270,7 @@ static bool write_pidfile(void)
 bool kill_other(int signal)
 {
 #ifndef HAVE_MINGW
-	int pid;
+	pid_t pid;
 
 	cp();
 
@@ -362,25 +356,29 @@ bool detach(void)
 bool execute_script(const char *name, char **envp)
 {
 #ifdef HAVE_SYSTEM
-	int status;
+	int status, len;
 	struct stat s;
 	char *scriptname;
 
 	cp();
 
 #ifndef HAVE_MINGW
-	asprintf(&scriptname, "%s/%s", confbase, name);
+	len = asprintf(&scriptname, "\"%s/%s\"", confbase, name);
+#else
+	len = asprintf(&scriptname, "\"%s/%s.bat\"", confbase, name);
+#endif
+	if(len < 0)
+		return false;
+
+	scriptname[len - 1] = '\0';
 
 	/* First check if there is a script */
 
-	if(stat(scriptname, &s))
+	if(stat(scriptname + 1, &s))
 		return true;
 
 	ifdebug(STATUS) logger(LOG_INFO, _("Executing script %s"), name);
 
-	free(scriptname);
-#endif
-
 #ifdef HAVE_PUTENV
 	/* Set environment */
 	
@@ -388,7 +386,7 @@ bool execute_script(const char *name, char **envp)
 		putenv(*envp++);
 #endif
 
-	asprintf(&scriptname, "\"%s/%s\"", confbase, name);
+	scriptname[len - 1] = '\"';
 	status = system(scriptname);
 
 	free(scriptname);
