@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: tincd.c,v 1.10.4.85 2003/08/08 22:11:54 guus Exp $
+    $Id: tincd.c,v 1.10.4.89 2003/10/06 16:13:08 guus Exp $
 */
 
 #include "system.h"
@@ -39,6 +39,7 @@
 #include <lzo1x.h>
 
 #include <getopt.h>
+#include <pidfile.h>
 
 #include "conf.h"
 #include "device.h"
@@ -291,7 +292,7 @@ static bool keygen(int bits)
 	char *filename;
 
 	fprintf(stderr, _("Generating %d bits keys:\n"), bits);
-	rsa_key = RSA_generate_key(bits, 0xFFFF, indicator, NULL);
+	rsa_key = RSA_generate_key(bits, 0x10001, indicator, NULL);
 
 	if(!rsa_key) {
 		fprintf(stderr, _("Error during key generation!\n"));
@@ -358,7 +359,7 @@ static void make_names(void)
 #ifdef HAVE_MINGW
 	if(!RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\tinc", 0, KEY_READ, &key)) {
 		if(!RegQueryValueEx(key, NULL, 0, 0, installdir, &len)) {
-			if(!pidfilename)
+			if(!logfilename)
 				asprintf(&logfilename, "%s/log/%s.log", identname);
 			if(!confbase) {
 				if(netname)
@@ -506,5 +507,10 @@ int main2(int argc, char **argv)
 		dump_device_stats();
 
 	logger(LOG_NOTICE, _("Terminating"));
+
+#ifndef HAVE_MINGW
+	remove_pid(pidfilename);
+#endif
+	
 	return status;
 }
